shuffle = function(in_data)
{
  data_shuffle = in_data
  for(i in 1:nrow(in_data))
  {
	nahodne_cislo = round(runif(1,min=1,max=nrow(in_data)))
	temp = data_shuffle[i,]
	data_shuffle[i,] = data_shuffle[nahodne_cislo,]
	data_shuffle[nahodne_cislo,] = temp
  }
  shuffle = data_shuffle
}


select_kal = function(in_data,n_kal)
{
  kal_points = in_data[1:n_kal,]
  select_kal = kal_points
}


select_val = function(in_data,n_kal)
{
  val_points = in_data[(n_kal+1):nrow(in_data),]
  select_val = val_points	
}


select_location = function(in_data,nth_row)
{
  location = vector(mode ='numeric',length=2)
  for(l in 1:2)
  {
    	location[l] = in_data[nth_row,l]
  }
  select_location = location
}


select_square = function(in_data, a, buffer, k_x, k_y, min_x, max_x, min_y, max_y)
{
  data_in_square = subset(in_data, (in_data[,1] >= min_x + k_x*a - buffer) & (in_data[,1] < min_x + (k_x+1)*a + buffer) & (in_data[,2] >= min_y + k_y*a - buffer) & (in_data[,2] < min_y + (k_y+1)*a + buffer))
  select_square = data_in_square
}


set_radius = function(in_data,n)
{
  n_all = nrow(in_data)
  x_min = min(in_data$x_coord)
  x_max = max(in_data$x_coord)
  y_min = min(in_data$y_coord)
  y_max = max(in_data$y_coord)
  area = (x_max-x_min)*(y_max-y_min)

  radius = 6*sqrt(n*area)/n_all
  set_radius = radius
}


select_circle = function(in_data,location,radius) 
{
  data_in_radius = subset(in_data, (in_data[,1]-location[1])^2 + (in_data[,2]-location[2])^2 < radius^2) 
  select_circle = data_in_radius
}


dist = function(in_data, location)
{
  nn = nrow(in_data)
  distances = matrix(nrow=nn, ncol=5)
  for (i in 1:nn) {
	distances[i,4] = sqrt((in_data[i,1]-location[1])^2 +(in_data[i,2]-location[2])^2)
	distances[i,1] = in_data[i,1]
	distances[i,2] = in_data[i,2]
	distances[i,3] = in_data[i,3]	
  }
  distances[,5] = rep(1:nn)
  dist = distances
}


bubble = function(distances)
{
  sorted_dist <- distances
  nn = nrow(sorted_dist)
  for(i in 1:nn)
  {
	for(j in 1:nn)
	{
		if(sorted_dist[i,4] <= sorted_dist[j,4])
		{
			temp = sorted_dist[i,]
			sorted_dist[i,] = sorted_dist[j,]
			sorted_dist[j,] = temp
		}
	}
  }
  bubble = sorted_dist
}


find_n_nearest = function(in_data,n,location,crossval,max_radius)	
{
  radius = set_radius(in_data,n)
  data_in_radius = select_circle(in_data,location,radius)    
  while((nrow(data_in_radius) < n+crossval) & (radius < max_radius))
  {
    radius = radius + 20
    data_in_radius = select_circle(in_data,location,radius)
  }  
  if(nrow(data_in_radius)>=n+crossval)
  {
    d = dist(data_in_radius,location)
    sorted_dist = bubble(d)
    n_nearest_pts = sorted_dist[(1+crossval):(n+crossval),]
  } else
  {
    n_nearest_pts = matrix(NA,nr=1)
  }
  find_n_nearest = n_nearest_pts
}


angle = function(in_data,location)
{
  angle360 = vector(mode='numeric',length=nrow(in_data))
  for(i in 1:nrow(in_data))
  {
	delta_x = in_data[i,1]-location[1]
	delta_y = in_data[i,2]-location[2]
	angle90 = abs(atan(delta_x/delta_y))*180/pi	# uhel v ramci sveho kvadrantu

	if((delta_x>=0) & (delta_y>=0))
	{
		angle360[i] = angle90
	} else
	{
		if((delta_x>=0) & (delta_y<0))
		{
			angle360[i] = 180-angle90
		} else
		{
			if((delta_x<0) & (delta_y<0))
			{
				angle360[i] = 180+angle90
			} else
			{
				angle360[i] = 360-angle90
			}
		}
	}
  }
  angle = angle360
}


make_grid = function(in_data,grid_resolution,crossval)
{
  if(crossval == 0)
  {
    x_min = min(in_data$x_coord)
    x_max = max(in_data$x_coord)
    y_min = min(in_data$y_coord)
    y_max = max(in_data$y_coord)

    i = 2
    x_coord=c()
    y_coord=c()

    x_coord[1] = x_min
    y_coord[1] = y_min

    repeat
    {
	repeat
	{
		x_coord[i] = x_coord[i-1] + grid_resolution
		y_coord[i] = y_coord[i-1]
		i = i+1
		if(x_coord[i-1] >= x_max) {break}
	}
	x_coord[i] = x_min
	y_coord[i] = y_coord[i-1] + grid_resolution
	i = i+1
	if(y_coord[i-1] > y_max) {break}
    }
    z_predict = c(rep(NA,length(x_coord)))
    out_grid = data.frame(x_coord,y_coord,z_predict)
    out_grid = out_grid[1:(nrow(out_grid)-1),]
  } else
  {
    z_predict = c(rep(NA,nrow(in_data)))
    out_grid = data.frame(in_data,z_predict)
  }
  make_grid = out_grid
}


make_patterns = function(in_data, out_grid, n, crossval, max_radius)          
{
  vzory = matrix(nrow=nrow(out_grid), ncol=2+5*n)

  for(i in 1:nrow(out_grid))
  {
    location = select_location(out_grid,i)
    l = length(location)
    n_nearest_pts = find_n_nearest(in_data,n,location,crossval,max_radius)         
    if(nrow(n_nearest_pts) >= n)
    {
	angle360 = angle(n_nearest_pts,location)

	vzory[i,1:l] = location
	pom_prom = l+1
	for(k in 1:n)
	{
		vzory[i,pom_prom] = n_nearest_pts[k,1]
		vzory[i,pom_prom+1] = n_nearest_pts[k,2]
		vzory[i,pom_prom+2] = n_nearest_pts[k,4]
		vzory[i,pom_prom+3] = angle360[k]
		vzory[i,pom_prom+4] = n_nearest_pts[k,3]
		pom_prom = pom_prom + 5
	}
    }
  }
  make_pattern = vzory
}


matrix_of_dist = function(in_data)
{
  N = nrow(in_data)
  matice_vzdalenosti = matrix(nrow=N,ncol=N)
  a = 1
  for(i in 1:N)
  {
	for(j in 1:N)
	{
		matice_vzdalenosti[i,j] = sqrt((in_data[i,1]-in_data[j,1])^2 +(in_data[i,2]-in_data[j,2])^2)
		matice_vzdalenosti[j,i] = matice_vzdalenosti[i,j]
	}
  }
  matrix_of_dist = matice_vzdalenosti
}


vrgram = function(in_data,lag,cut)	
{
  N = nrow(in_data)
  matice_vzdalenosti = matrix(nrow=N,ncol=N)
  a = 1
  for(i in 1:N)
  {
	for(j in a:N)
	{
		matice_vzdalenosti[i,j] = sqrt((in_data[i,1]-in_data[j,1])^2 +(in_data[i,2]-in_data[j,2])^2)
		matice_vzdalenosti[j,i] = -1 
	}
  	a = a+1
  }

  max_dist = max(matice_vzdalenosti)
  if(max_dist > cut)
  {
    max_dist = cut
  }
  
  n_row=round(max_dist/lag)

  LAG = vector(mode='numeric',length=n_row)
  N_LAG = vector(mode='numeric',length=n_row)
  semivar_standard = vector(mode='numeric',length=n_row)
  semivar_Cressie1 = vector(mode='numeric',length=n_row)
  semivar_Cressie2 = vector(mode='numeric',length=n_row)

  for(k in 1:n_row)
  {
	H = k*lag
	indexy = which((matice_vzdalenosti > H-lag) & (matice_vzdalenosti <= H), arr.ind = TRUE)
	N_H = nrow(indexy)
	LAG[k] = H-lag/2	
	N_LAG[k] = N_H
	semivar_standard[k] = 1/(2*N_H)*sum((in_data[indexy[,1],3]-in_data[indexy[,2],3])^2)
	semivar_Cressie1[k] =	((1/(N_H)*sum((abs(in_data[indexy[,1],3]-in_data[indexy[,2],3]))^0.5))^4)/(2*(0.457+0.494/N_H))
	semivar_Cressie2[k] = (median((abs(in_data[indexy[,1],3]-in_data[indexy[,2],3]))^0.5)^4)/(2*0.457)
  }
  semivars = data.frame(LAG,N_LAG,semivar_standard,semivar_Cressie1,semivar_Cressie2)
  vrgram = semivars
}


exp_vrgram = function(psill,nugget,range,LAG)	
{
  result = vector(mode='numeric', length=length(LAG))

  for (i in 1:length(LAG))
	{
		result[i] = nugget + psill*(1 - exp(-3*LAG[i]/range))
	}
  exp_vrgram = result
}


sph_vrgram = function(psill,nugget,range,LAG)
{
  result = vector(mode='numeric', length=length(LAG))

  for (i in 1:length(LAG))
  {
	if (LAG[i] <= range)
	{
		result[i] = nugget + (psill)*((3*LAG[i])/(2*range)-(LAG[i]^3)/(2*range^3))
	} else
	{	
		result[i] = nugget + (psill)
	}
  }
  sph_vrgram = result
}


gau_vrgram = function(psill,nugget,range,LAG)
{
  result = vector(mode='numeric', length=length(LAG))

  for (i in 1:length(LAG))
	{
		result[i] = nugget + psill*(1-exp(-3*LAG[i]^2 / range^2)) 
	}
  gau_vrgram = result
}


pow_vrgram = function(nugget,range,LAG,pow)
{
  result = vector(mode='numeric', length=length(LAG))

  for (i in 1:length(LAG))
	{
		result[i] = nugget + range*LAG[i]^pow 
	}
  pow_vrgram = result
}      


fit_vrgram = function(semivars,LAG,data_var,type_of_theoretical_variogram,psill_ini,range_ini,pow_ini)
{
  if(type_of_theoretical_variogram == 1)
  {
  	exp_variogram = nls(semivars~exp_vrgram(psill, nugget, range, LAG), data=data_var, start = c(psill=psill_ini, range=range_ini), trace=TRUE)
	preds = predict(exp_variogram)
  } else
  {
	if(type_of_theoretical_variogram == 2)
	{
	  sph_variogram = nls(semivars~sph_vrgram(psill, nugget, range, LAG), data=data_var, start = c(psill=psill_ini, range=range_ini), trace=TRUE)
	  preds = predict(sph_variogram)
	} else
	{
		if(type_of_theoretical_variogram == 3)
		{
		  gau_variogram = nls(semivars~gau_vrgram(psill, nugget, range, LAG), data=data_var, start = c(psill=psill_ini, range=range_ini), trace=TRUE)
		  preds = predict(gau_variogram)
		} else
		{
		  pow_variogram = nls(semivars~pow_vrgram(nugget, range, LAG, pow), data=data_var, start = c(range=range_ini, pow=pow_ini), trace=TRUE)
		  preds = predict(pow_variogram)
		}
	}
  }
  fit_vrgram = preds
}


graph_variogram = function(LAG,semivars,type_of_experimental_variogram,type_of_theoretical_variogram,nugget,preds,title)
{
  color = c('blue','red','green')
  legend_e_var = c("Experimental: conventional", "Experimental: Cressie & Hawkins based on mean", "Experimental: Cressie & Hawkins based on median")
  legend_t_var = c('Theoretical: exponential', 'Theoretical: spherical', 'Theoretical: gaussian', 'Theoretical: power')

  maxLAG = max(LAG,na.rm=TRUE)
  oLAG = round(log(max(LAG,na.rm=TRUE)-min(LAG,na.rm=TRUE),10)) - 1	
  abline_v = 10^oLAG

  maxSemivar = max(semivars,na.rm=TRUE)
  oSemivar = round(log(max(semivars,na.rm=TRUE)-min(semivars,na.rm=TRUE),10)) - 1
  abline_h = 10^oSemivar

  plot(LAG, semivars,
	type='p', col=color[type_of_experimental_variogram], xlim=c(0,maxLAG*1.05), ylim=c(0,maxSemivar*1.3),
	xlab='Lag (m)', ylab='Semivariance', lwd=2, main = title, sub = 'Variograms')

	lines(c(0,LAG), c(nugget,preds), col=color[type_of_experimental_variogram], lwd=2, lty=5)

	abline(v = seq(0, maxLAG+10, by = abline_v), h = seq(0, maxSemivar*1.3, by = abline_h),
		col = "darkgray", lty=2, lwd = .5)
	
	legend("topleft", legend = c(legend_e_var[type_of_experimental_variogram],
		legend_t_var[type_of_theoretical_variogram]),
		col=c(color[type_of_experimental_variogram],color[type_of_experimental_variogram]),
		pch=1, pt.lwd=2, xjust = 1, yjust = 1, bty != 'n', bg = "white",
		title = "Semivariograms")
}


dist2gamma = function(LAG,lag,n_nearest_pts,preds)
{
  gamma = vector(mode='numeric',length=nrow(n_nearest_pts))
  for(i in 1:length(LAG))
  {
	indexy = which((n_nearest_pts[,4] > LAG[i]-lag/2) & (n_nearest_pts[,4] <= LAG[i]+lag/2), arr.ind = TRUE)
	gamma[indexy] = preds[i]
  }
  dist2gamma = gamma
}


dist2GAMMA = function(LAG,lag,matice_vzdalenosti,preds)
{
  GAMMA = matrix(nr=nrow(matice_vzdalenosti),nc=ncol(matice_vzdalenosti))
  for(i in 1:length(LAG))
  {
	indexy = which((matice_vzdalenosti > LAG[i]-lag/2) & (matice_vzdalenosti <= LAG[i]+lag/2), arr.ind = TRUE)
	if(nrow(indexy)>0)
	{	
		for(j in 1:nrow(indexy))
		{
		  GAMMA[indexy[j,1],indexy[j,2]] = preds[i]
		}
	}
  }
  for(i in 1:nrow(GAMMA))
  {
	GAMMA[i,i] = 0
  }
  dist2GAMMA = GAMMA
}


map_sp = function(out_grid,title)
{
  library(sp)
  library(lattice)
  trellis.par.set(sp.theme())

  coordinates(out_grid) = ~x_coord + y_coord

  o = round(log(max(out_grid$z_predict,na.rm=TRUE)-min(out_grid$z_predict,na.rm=TRUE),10)) - 1
  
  spplot(out_grid, 'z_predict', do.log=TRUE,
    key.space = 'right',
    scales=list(draw=T),
    cuts = seq(round(min(out_grid$z_predict,na.rm=TRUE)-1,o), round(max(out_grid$z_predict,na.rm=TRUE)+1,o), by=10^o),
    main = title)
}


map_rgl = function(out_grid,title)
{
  library(rgl)
  
  z_range <- max(out_grid$z_predict, na.rm = TRUE) - min(out_grid$z_predict, na.rm = TRUE) + 1
  colormap <- terrain.colors(z_range)
  color <- colormap[out_grid$z_predict]
  
  rgl.points(out_grid$x_coord, out_grid$y_coord, out_grid$z_predict, col = color)

  x = subset(out_grid$y_coord, out_grid$x_coord == min(out_grid$x_coord, na.rm = TRUE))
  y = subset(out_grid$x_coord, out_grid$y_coord == max(out_grid$y_coord, na.rm = TRUE))
  z = matrix(nc = length(y), nr = length(x))
  for(i in 1:nrow(z))
  {
    for(j in 1:ncol(z))
    {
      z[i,j] = out_grid$z_predict[(i-1)*ncol(z)+j]
    }
  }

  axes3d(edges = "bbox", labels = TRUE, tick = TRUE, nticks = 5)
  title3d(main = title, sub = NULL, xlab = '(m)', ylab = '(m)', zlab = '(m)')
  bg3d('white')
}


map_fields = function(out_grid,title)
{
  library(fields)

  x = subset(out_grid$y_coord, out_grid$x_coord == min(out_grid$x_coord, na.rm = TRUE))
  y = subset(out_grid$x_coord, out_grid$y_coord == max(out_grid$y_coord, na.rm = TRUE))
  z = matrix(nc = length(y), nr = length(x))
  for(i in 1:nrow(z))
  {
    for(j in 1:ncol(z))
    {
      z[i,j] = out_grid$z_predict[(i-1)*ncol(z)+j]
    }
  }
  
  drape.plot(x, y, z,
    phi = 90, theta = 90, box = TRUE,
    col = terrain.colors(128),
    border = NA,
    xlab = 'x', ylab = 'y', zlab = 'z',
    ltheta = 120, scale = TRUE, shade = 0.5, horizontal = TRUE, ticktype = 'detailed', expand = 0.5,
    add.legend = TRUE)
}


idw = function(in_data,out_grid,n,r,crossval,max_radius)      
{
  for(i in 1:nrow(out_grid))
  {
    location = select_location(out_grid,i)
    n_nearest_pts = find_n_nearest(in_data,n,location,crossval,max_radius)     
    if(nrow(n_nearest_pts) >= n)
    {
	z = n_nearest_pts[,3]
	d = n_nearest_pts[,4]
	
	if(d[1] == 0)
	{
	  z_pred = z[1]
	} else
	{
	  z_pred = sum(z*d^(-r))/sum(d^(-r))
	}
	out_grid$z_predict[i] = round(z_pred,3)
    }
  }
  idw = out_grid
}


variogram = function(in_data, lag, cut, nugget, type_of_experimental_variogram, type_of_theoretical_variogram, title)  # variogram = vrgram + fit_vrgram
{
  experiment_variogram_all = vrgram(in_data,lag,cut)
  experiment_variogram = subset(experiment_variogram_all, experiment_variogram_all$N_LAG > 0)
  LAG = experiment_variogram$LAG
  semivars = experiment_variogram[,2+type_of_experimental_variogram]
  data_var = data.frame(LAG,semivars)

  psill_ini = 0.5*max(semivars)-nugget
  range_ini = 0.5*max(LAG)
  pow_ini = 1
  preds = fit_vrgram(semivars,LAG,data_var,type_of_theoretical_variogram,psill_ini,range_ini,pow_ini)
  
  pdf(paste(out_dir,'variogram.pdf',sep=''))
  graph_variogram(LAG,semivars,type_of_experimental_variogram,type_of_theoretical_variogram,nugget,preds,title)	
  dev.off()
  
  variogram = preds
}


ok = function(in_data, out_grid, lag, cut, nugget, type_of_experimental_variogram, type_of_theoretical_variogram, n, r, crossval, title, max_radius)    
{
  experiment_variogram_all = vrgram(in_data,lag,cut)
  experiment_variogram = subset(experiment_variogram_all, experiment_variogram_all$N_LAG > 0)
  LAG = experiment_variogram$LAG
  semivars = experiment_variogram[,2+type_of_experimental_variogram]
  data_var = data.frame(LAG,semivars)

  psill_ini = 0.5*max(semivars)-nugget
  range_ini = 0.5*max(LAG)
  pow_ini = 1
  preds = fit_vrgram(semivars,LAG,data_var,type_of_theoretical_variogram,psill_ini,range_ini,pow_ini)
  
  z_var = vector(mode='numeric', length=nrow(out_grid))
  out_grid = data.frame(out_grid,z_var)
 
  for(i in 1:nrow(out_grid))
  {
    location = select_location(out_grid,i)
    n_nearest_pts = find_n_nearest(in_data,n,location,crossval,max_radius)     
    if(nrow(n_nearest_pts) >= n)
    {
	matice_vzdalenosti = matrix_of_dist(n_nearest_pts)
	gamma = dist2gamma(LAG,lag,n_nearest_pts,preds)
	GAMMA = dist2GAMMA(LAG,lag,matice_vzdalenosti,preds)  
	ones = c(rep(1,length(gamma)))

	z_pred = (gamma + ones * ((1 - t(ones) %*% solve(GAMMA) %*% gamma)/(t(ones) %*% solve(GAMMA) %*% ones))) %*% solve(GAMMA) %*% n_nearest_pts[,3]

	z_var = t(gamma) %*% solve(GAMMA) %*% gamma - (t(ones) %*% solve(GAMMA) %*% gamma - 1)^2/(t(ones) %*% solve(GAMMA) %*% ones)

	out_grid$z_predict[i] = round(z_pred,3)
	out_grid$z_var[i] = round(z_var,3)
    }
  }

  pdf(paste(out_dir,'variogram.pdf',sep=''))
  graph_variogram(LAG,semivars,type_of_experimental_variogram,type_of_theoretical_variogram,nugget,preds,title)	
  dev.off()

  ok = out_grid
}


uk = function(in_data, out_grid, lag, cut, nugget, type_of_experimental_variogram, type_of_theoretical_variogram, n, r, crossval, title, max_radius)    
{
  experiment_variogram_all = vrgram(in_data,lag,cut)
  experiment_variogram = subset(experiment_variogram_all, experiment_variogram_all$N_LAG > 0)
  LAG = experiment_variogram$LAG
  semivars = experiment_variogram[,2+type_of_experimental_variogram]
  data_var = data.frame(LAG,semivars)

  psill_ini = 0.7*max(semivars)-nugget
  range_ini = 0.3*max(LAG)
  pow_ini = 1
  preds = fit_vrgram(semivars,LAG,data_var,type_of_theoretical_variogram,psill_ini,range_ini,pow_ini)
  
  z_var = vector(mode='numeric', length=nrow(out_grid))
  out_grid = data.frame(out_grid,z_var)
 
  for(i in 1:nrow(out_grid))
  {
    location = select_location(out_grid,i)
    n_nearest_pts = find_n_nearest(in_data,n,location,crossval,max_radius)      
    if(nrow(n_nearest_pts) >= n)
    {
	matice_vzdalenosti = matrix_of_dist(n_nearest_pts)
	gamma = dist2gamma(LAG,lag,n_nearest_pts,preds)
	GAMMA = dist2GAMMA(LAG,lag,matice_vzdalenosti,preds)  
	X = n_nearest_pts[,1:2]

	z_pred = t(gamma + X %*% solve(t(X) %*% solve(GAMMA) %*% X) %*% (location - t(X) %*% solve(GAMMA) %*% gamma)) %*% solve(GAMMA) %*% n_nearest_pts[,3]

	out_grid$z_predict[i] = round(z_pred,3)
	out_grid$z_var[i] = round(z_var,3)
    }
  }

  pdf(paste(out_dir,'variogram.pdf',sep=''))
  graph_variogram(LAG,semivars,type_of_experimental_variogram,type_of_theoretical_variogram,nugget,preds,title)	
  dev.off()

  uk = out_grid
}